# -*- coding: utf-8 -*-
from functools import wraps
from .http import smart_response


def uturn(view):
    """
    Decorator for view functions that applies smart redirects when needed.

    If the view responds with a redirect, this decorator will check whether
    a *better* redirect location is specified in the request and if there is,
    override the redirect with the new location.

    This only applies to temporary redirects, not permanent redirects or any
    other kind of response.

    """
    @wraps(view)
    def wrap(request, *args, **kwargs):
        response = view(request, *args, **kwargs)
        return smart_response(request, response)
    return wrap



# -*- coding: utf-8 -*-
try:
    import urlparse
except ImportError:
    # Python 3
    import urllib.parse as urlparse

from django.shortcuts import redirect as core_redirect
from django.http import HttpResponseRedirect
from django.utils.encoding import iri_to_uri
from django.conf import settings


def param_name():
    return getattr(settings, 'UTURN_REDIRECT_PARAM', 'next')


def get_redirect_url(request):
    """
    Retrieves the next URL from the request.

    The next URL parameter is identified by the ``UTURN_REDIRECT_PARAM``,
    which defaults to ``next``. If the parameter is present and is a
    relative URL, it will be returned. In any other case, ``None`` is
    returned.

    Only URLs pointing to whitelisted domains (specified in the setting
    ``UTURN_ALLOWED_HOSTS``) and the current domain are allowed to prevent
    redirects to untrusted sites. Have a look at
    `URL Redirection to Untrusted site
    <http://cwe.mitre.org/data/definitions/601.html>`_ to discover the risks
    involved.

    """
    if request is None:
        return None
    next = None
    param = param_name()
    if request.method == 'GET':
        next = request.GET.get(param, None)
    elif request.method == 'POST':
        next = request.POST.get(param, None)
    if not next:
        return None
    # Check if it's an absolute URL.
    allowed_hosts = getattr(settings, 'UTURN_ALLOWED_HOSTS', None)
    allowed_hosts = allowed_hosts if allowed_hosts else [request.get_host()]
    host = urlparse.urlparse(next)[1]
    if host:
        # Make sure the absolute URL points to an allowed host, otherwise
        # ignore the value.
        return next if host in allowed_hosts else None
    # Make sure it's a relative URL to prevent "open redirects"
    return next if next.startswith('/') else None


def smart_redirect(request, to, *args, **kwargs):
    """
    Either redirects the user like Django's ``redirect`` shortcut would, or
    in case a redirect url is present, redirects to that instead.

    The next parameter takes precedence.

    """
    next = get_redirect_url(request)
    if next:
        return core_redirect(next)
    return core_redirect(to, *args, **kwargs)


def smart_response(request, response):
    """
    Reissues a redirect when necessary - leaves other response alone.

    The response is examined; in case it's a temporary redirect and a
    redirect alternative is specified in the request, a new redirect is
    constructed targeting the alternative. In all other cases, the response
    is returned as is.

    """
    if not response:
        return response
    location = response.get('Location', None)
    if not location or response.status_code != 302:
        return response
    next = get_redirect_url(request)
    if next:
        response['Location'] = iri_to_uri(next)
    return response


class SmartHttpResponseRedirect(HttpResponseRedirect):
    """
    Acts like Django's regular ``HttpResponseRedirect``, unless a redirect
    alternative is discovered in the request parameters.

    """

    def __init__(self, request, redirect_to):
        next = get_redirect_url(request)
        redirect_to = next if next else redirect_to
        super(SmartHttpResponseRedirect, self).__init__(redirect_to)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django import template, VERSION as django_version
from django.template.defaulttags import URLNode
from django.utils.http import urlencode
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from uturn.http import get_redirect_url, param_name


register = template.Library()


class DefaultUrlNode(URLNode):
    """
    Duplicate of the standard Django URL node that will render the "uturn"
    parameter from the request or if no such parameter exists, the URL supplied
    to the tag.
    
    It assumes the Context is a RequestContext. Make sure you've added the 
    following template context processor in your settings as this provides
    access to the request from within the template tag::

        TEMPLATE_CONTEXT_PROCESSORS += (
            'django.core.context_processors.request',
        )
    
    An example. Usually you'd use the following to render a link::
    
        {% url viewname id %}
    
    If you want to give the optional ``uturn`` request parameter precedence 
    over the normal url, you'd use this::
    
        {% defaulturl viewname id %}
        
    Now when you visit the page with the above template, you'll either have
    the same link that results from the normal url template tag, or, when a
    valid uturn parameter is present, a link to the value of the uturn 
    parameter.
    """
    def __init__(self, url_node):
        self.url_node = url_node
    
    def render(self, context):
        next = get_redirect_url(context.get('request', None)) 
        url = next if next else self.url_node.render(context)
        if self.url_node.asvar:
            context[self.url_node.asvar] = url
            return ''
        return url


class UturnUrlNode(URLNode):
    """
    Duplicate of the standard Django URL node that will add the current request
    path to the URL.
    
    The page at the given link will then receive the ``uturn`` request 
    parameter, which means you can redirect the user back to the page 
    containing the uturn template tag, overriding the default redirect.

    It assumes the Context is a RequestContext. Make sure you've added the 
    following template context processor in your settings as this provides
    access to the request from within the template tag::

        TEMPLATE_CONTEXT_PROCESSORS += (
            'django.core.context_processors.request',
        )
    
    An example. Usually you'd use the following to render a link::
    
        {% url viewname id %}
    
    Suppose that page contains a form and you want the user to be redirected
    to the current page rather than following the default redirect::
    
        {% uturn viewname id %}
        
    Of course, the view should use the Uturn redirect mechanisms for this to
    work. If you have a "Cancel" link on the form to take the user back to
    the previous page, use the ``defaulturl`` template tag::

        {% defaulturl previousviewname id %}
    """
    def __init__(self, url_node):
        self.url_node = url_node
    
    def render(self, context):
        url = self.url_node.render(context)
        next = get_redirect_url(context.get('request', None))
        if next:
            param = param_name()
            part = urlencode({param: next})
            sep = '&' if url.find('?') >= 0 else '?'
            url = url + sep + part
        if self.url_node.asvar:
            context[self.url_node.asvar] = url
            return ''
        return url


@register.tag(name='defaulturl')
def do_default_url(parser, token):
    """
    Renders the supplied url or, in case a valid ``next`` parameter was 
    passed, the value of the ``next`` parameter.

    This template tag behaves exactly like Django's default ``url`` tag when
    no ``next`` url was found in the request.

    """
    node = template.defaulttags.url(parser, token)
    return DefaultUrlNode(node)


@register.tag(name='uturn')
def do_uturn(parser, token):
    """
    Renders the supplied url while adding the current request path to the query
    string.

    This template tag behaves exactly like Django's default ``url`` tag. It 
    just adds the current full request path to the url as the ``next``
    parameter.
    
    """
    node = template.defaulttags.url(parser, token)
    return UturnUrlNode(node)


def _do_uturn_param(request):
    next = get_redirect_url(request)
    if next:
        attr = {
            'param': conditional_escape(param_name()),
            'value': conditional_escape(next)
        }
        f = "<input type='hidden' name='%(param)s' value='%(value)s'>" % attr
        return mark_safe("<div style='display:none'>%s</div>" % f)
    return ''    


# This is ugly stuff, but if this is all it takes to support Django 1.2, it'll
# have to do.
if django_version[:2] == (1, 2):
    @register.simple_tag
    def uturn_param(request):
        return _do_uturn_param(request)
else:
    @register.simple_tag(takes_context=True)
    def uturn_param(context):
        return _do_uturn_param(context.get('request', None))



# -*- coding: utf-8 -*-
from .http import smart_response


class UturnMiddleware(object):
    """
    Middleware that applies the logic of the ``uturn`` decorator to each
    request.

    If you want to enable Uturn redirects on each request, add this middleware
    class to your Django settings. If you prefer to explicitly specify which
    views can use uturn redirects, use the ``uturn.views.uturn`` decorator.

    """
    def process_response(self, request, response):
        return smart_response(request, response)



# -*- coding: utf-8 -*-
from distutils.core import setup

setup(
    name='django-uturn',
    version='0.2.3',
    author='Kevin Wetzels',
    author_email='kevin@roam.be',
    url='https://github.com/roam/django-uturn',
    packages=['uturn', 'uturn.templatetags'],
    license='BSD licence, see LICENCE',
    description='Overriding redirects in Django, to return where you came '\
                'from',
    long_description=open('README.rst').read(),
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
   ],
)